unit eBob42.WebSpinEdit;
// To create a more advanced Web Control that supports live data at
// design time, see instructions in the readme file located in the
// 'BDS\3.0\Source\dotNet\dbwebcontrols' directory

interface
{$AUTOBOX ON}
uses
  System.Web.UI, System.Web.UI.WebControls, eBob42.NicerLabelControl,
  System.ComponentModel;

type
  /// <summary>
  /// Summary description for MyWebControl1.
  /// </summary>
  [DefaultProperty('Text'),
   ToolboxData('<{0}:WebSpinEdit runat=server></{0}:WebSpinEdit>')]
  WebSpinEdit = class(System.Web.UI.WebControls.WebControl, INamingContainer)
  strict private
    FText: String;
    FUpButton, FDownButton: Button;
    FTextBox: TextBox;
    FLabel: NicerLabelControl;
  strict protected
    // override RenderContents(..) when you inherit from Web.UI.WebControls.WebControl
    procedure RenderContents(Output: HtmlTextWriter); override;
    procedure CreateChildControls; override;

   // override Render(..) when you inherit from Web.UI.WebControls.Control
   // procedure Render(Output: HtmlTextWriter); override;
  public
    constructor Create;
    function get_Value: Integer;
    procedure set_Value(const Value: Integer);
    procedure UpClick(sender: TObject; e: EventArgs);
    procedure DownClick(sender: TObject; e: EventArgs);
  published
    [Bindable(true),
     Category('Appearance'),
     DefaultValue('')]
    property Text: string read FText write FText;

    [Category('eBob42')]
    property Value: Integer read get_Value write set_Value;
  end;

implementation

uses
  System.Drawing;

/// <summary>
/// Define a public parameterless constructor needed by web controls.
/// </summary>
constructor WebSpinEdit.Create;
begin
  inherited;
end;

/// <summary>
/// Render this control to the output parameter specified, preserving
/// cosmetic attribute output generation inherited from standard WebControl.
/// </summary>
/// <param name="output"> The HTML writer to write out to </param>
procedure WebSpinEdit.CreateChildControls;
begin
  inherited;
  FUpButton := Button.Create;
  FupButton.Text := 'Up';
  include(FUpButton.Click, upClick);
  Controls.Add(FUpButton);
  FDownButton := BUtton.Create;
  FDownButton.Text := 'Down';
  include(FDownButton.Click, DownClick);
  Controls.Add(FDownButton);
  FTextBox := TextBox.Create;
  FTextBox.Text := Value.ToString;
  Controls.Add(FTextBox);
  FLabel := NicerLabelControl.Create;
  FLabel.ForeColor := Color.Red;
  FLabel.ForeColorEnd := Color.Blue;
  FLabel.Text := Text;
  Controls.Add(FLabel);
end;

procedure WebSpinEdit.DownClick(sender: TObject; e: EventArgs);
begin
  Value := Value - 1;
end;

function WebSpinEdit.get_Value: Integer;
begin
  if Assigned(ViewState['WebCheckBoxValue']) then
    Result := ViewState['WebCheckBoxValue'] as Integer
  else
    Result := 42;
end;

procedure WebSpinEdit.RenderContents(Output: HtmlTextWriter);
begin
  EnsureChildControls;
  FLabel.RenderControl(Output);
  Output.Write(': ');
  FTextBox.RenderControl(Output);
  FUpButton.RenderControl(Output);
  FDownButton.RenderControl(Output);
end;

{$REGION 'Render override'}
(*
/// <summary>
/// Render this control to the output parameter specified.
/// </summary>
/// <param name="output"> The HTML writer to write out to </param>
procedure MyWebControl1.Render(Output: HtmlTextWriter);
begin
  Output.Write(Text);
end;
(* *)
{$ENDREGION}

procedure WebSpinEdit.set_Value(const Value: Integer);
begin
  EnsureChildControls;
  ViewState['WebCheckBoxValue'] := Value;
  FTextBox.Text := Value.ToString;
end;

procedure WebSpinEdit.UpClick(sender: TObject; e: EventArgs);
begin
  Value := Value + 1;
end;

end.
