unit Oblong.ShadingLabel;

interface

uses
  SysUtils, Classes, Controls, StdCtrls;

type
  TLabelStyle = (lsLowered, lsNormal, lsRaised);

  TShadingLabel = class(TLabel)
  private
    FLabelStyle: TLabelStyle;
    FOffset: Integer;
    procedure SetLabelStyle(const Value: TLabelStyle);
    procedure SetOffset(const Value: Integer);
    { Private declarations }
  protected
    procedure Paint; override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    { Public declarations }
  published
    property LabelStyle: TLabelStyle read FLabelStyle write SetLabelStyle default lsLowered;
    property Offset: Integer read FOffset write SetOffset default 1;
  end;

implementation

uses
  Windows, Graphics, Types;

{ TShadingLabel }

constructor TShadingLabel.Create(AOwner: TComponent);
begin
  inherited;
  FOffset := 1;
  FLabelStyle := lsLowered;
  Transparent := True
end;

procedure TShadingLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Len: Integer; // Caption Length
  CalcOffset: Integer;
  OldColor: TColor;
  TempRect: TRect;
begin
  Len := Length(Caption);
  CalcOffset := Pred(Ord(FLabelStyle)) * FOffset;
  if CalcOffset <> 0 then
    try
      OldColor := Font.Color;
      Canvas.Brush.Style := bsClear;
      Canvas.Font := Self.Font;
      Canvas.Font.Color := clWhite; // white light
      TempRect := Rect(ClientRect.Left - CalcOffset, ClientRect.Top - CalcOffset,
                       ClientRect.Right - CalcOffset,
                       ClientRect.Bottom - CalcOffset);
      DrawText(Canvas.Handle, Caption, Len, TempRect,
        DT_EXPANDTABS or DT_WORDBREAK or Alignments[Alignment]);
      Canvas.Font.Color := clBlack; // black shadow
      TempRect := Rect(ClientRect.Left + CalcOffset, ClientRect.Top + CalcOffset,
                       ClientRect.Right + CalcOffset,
                       ClientRect.Bottom + CalcOffset);
      DrawText(Canvas.Handle, Caption, Len, TempRect,
        DT_EXPANDTABS or DT_WORDBREAK or Alignments[Alignment])
    finally
      Canvas.Font.Color := OldColor
    end;
  inherited; // paint original Caption
end;

procedure TShadingLabel.SetLabelStyle(const Value: TLabelStyle);
begin
  if FLabelStyle <> Value then
  begin
    FLabelStyle := Value;
    Invalidate
  end
end;

procedure TShadingLabel.SetOffset(const Value: Integer);
begin
  if FOffset <> Value then
  begin
    FOffset := Value;
    Invalidate
  end
end;

end.
