unit eBob42.PropertyEditors;
interface
uses
  Classes,
  {$IFDEF CLR}
    Borland.Vcl.Design.DesignIntf,
    Borland.Vcl.Design.DesignEditors;
  {$ELSE}
  DesignIntf,  DesignEditors;
  {$ENDIF}

type
  THelpFileProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

type
  TDatePickerProperty = class(TDateProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

type
  TMultiLineStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

type
  TDateFieldProperty = class(TStringProperty)
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

implementation
uses
  Forms, Controls, StdCtrls, ComCtrls, Dialogs, TypInfo, DB;


{ THelpFileProperty }

function THelpFileProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog]
end {GetAttributes};

procedure THelpFileProperty.Edit;
begin
  with TOpenDialog.Create(Application) do
  try
    Title := 'Dr.Bob''s ' + GetName + ' Property Editor Dialog';
    Filename := GetValue;
    DefaultExt := 'hlp';
    Filter := 'Help Files (*.hlp)|*.hlp';
    HelpContext := 0;
    Options := Options + [ofShowHelp, ofPathMustExist, ofFileMustExist];
    if Execute then SetValue(Filename);
  finally
    Free
  end
end {Edit};


{ TDateTimePickerProperty }

function TDatePickerProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog]
end;

procedure TDatePickerProperty.Edit;
var
  Form: TForm;
  Calendar: TMonthCalendar;
begin
  Form := TForm.Create(Application);
  with Form do
  try
    BorderStyle := bsToolWindow;
    Caption := (GetComponent(0) as TComponent).Name + '.' + GetName;
    ClientWidth := 169;
    Position := poMainFormCenter;
    Calendar := TMonthCalendar.Create(Form);
    with Calendar do
    begin
      Parent := Form;
      Align := alTop;
      Date := GetFloatValue
    end;
    with TButton.Create(Form) do
    begin
      Parent := Form;
      Caption := 'OK';
      ModalResult := mrOk;
      Default := True;
      Left := 8;
      Top := 8 + Calendar.Height
    end;
    with TButton.Create(Form) do
    begin
      Parent := Form;
      Caption := 'Cancel';
      ModalResult := mrCancel;
      Cancel := True;
      Left := 90;
      Top := 8 + Calendar.Height
    end;
    ClientHeight := 40 + Calendar.Height;
    if ShowModal = mrOk then SetFloatValue(Calendar.Date)
  finally
    Form.Free
  end
end;


{ TMultiLineStringProperty }

function TMultiLineStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog]
end {GetAttributes};

procedure TMultiLineStringProperty.Edit;
var
  Form: TForm;
  Memo: TMemo;
begin
  Form := TForm.Create(Application);
  with Form do
  try
    BorderStyle := bsSizeToolWin;
    Caption := (GetComponent(0) as TComponent).Name + '.' + GetName;
    ClientWidth := 200;
    Position := poMainFormCenter;
    Memo := TMemo.Create(Form);
    with Memo do
    begin
      Parent := Form;
      Align := alTop;
      ScrollBars := ssVertical;
      Text := Value
    end;
    with TButton.Create(Form) do
    begin
      Parent := Form;
      Caption := 'OK';
      ModalResult := mrOk;
      Default := True;
      Left := 8;
      Top := 8 + Memo.Height
    end;
    with TButton.Create(Form) do
    begin
      Parent := Form;
      Caption := 'Cancel';
      ModalResult := mrCancel;
      Cancel := True;
      Left := 108;
      Top := 8 + Memo.Height
    end;
    ClientHeight := 40 + Memo.Height;
    Constraints.MaxHeight := Height;
    Constraints.MinHeight := Height;
    Constraints.MinWidth := Width;
    if ShowModal = mrOk then SetValue(Memo.Text)
  finally
    Form.Free
  end
end {Edit};


{ TDateFieldProperty }

function TDateFieldProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TDateFieldProperty.GetValues(Proc: TGetStrProc);
var
  DataSource: TDataSource;
  DataSet: TDataSet;
  i: Integer;
begin
  DataSource := GetObjectProp(GetComponent(0), 'DataSource') as TDataSource;
  if Assigned(DataSource) then
  begin
    DataSet := DataSource.DataSet;
    if Assigned(DataSet) then
    begin
      if not DataSet.FieldDefs.Updated then DataSet.FieldDefs.Update;
      for i:=0 to DataSet.FieldDefs.Count-1 do
        if DataSet.FieldDefs[i].DataType in [ftDate, ftDateTime] then
          Proc(DataSet.FieldDefs[i].Name)
    end
  end
end;

end.
